-- Tacview ACMI - Universal Flight Analysis Tool 0.90
-- Export script for Lock On: Flaming Cliffs 1.1
-- Copyright (C) 2006-2007 - Stra Software
-- See http://lomac.strasoftware.com/lomac-acmi.php for more info

-- ACMI text files are exported in [/Lock On/Temp/] folder

-- TO ENABLE THIS SCRIPT:
--     Set [EnableExportScript = true] in [/Lock On/Config/export/config.lua]
--     Add [dofile("./Config/export/tacview.lua")] at the end of [/Lock On/Config/export/export.lua]

-- ACMI Log
AcmiData=
{
	-- Log parameters
	UpdatePeriod=1/10,			-- delay between two log updates (in seconds), use [0] to force update at each frame
	LatitudeOffset=42,			-- To improve log resolution
	LongitudeOffset=32,			-- To improve log resolution

	-- Reduce ID size (to compact text log)
	-- OptimizedID=ID^0x1000000  (free in C, hard to do with this fucking LUA...)
	GetOptimizedID=function(ID)

		if math.mod(math.floor(ID/16777216),2)==1 then
			return ID-16777216;	-- Clear bit
		end

		return ID+16777216;		-- Set bit
	end,

	-- Add escape characters
	CleanupName=function(Name)

		if Name then
			Name=string.gsub(Name,"\\","\\\\");
			return string.gsub(Name,",","\\,");
		end

		return Name;
	end,

	-- Begin to log ACMI data
	BeginLog=function(self)

		-- Reset objects list
		self.Objects={};

		-- Reset tools
		self.NextUpdateTime=0;

		-- Create a new log file for each mission
		self.AcmiFile=io.open("./Temp/Tacview-"..os.date("%Y%m%d-%H%M%S")..".txt.acmi","w");

		-- Write header
		if self.AcmiFile then

			-- Log ID
			self.AcmiFile:write("FileType=text/acmi/tacview\n");
			self.AcmiFile:write("FileVersion=1.0\n");
			self.AcmiFile:write("Source=Lock On: Flaming Cliffs 1.1\n");

			-- Mission info
			self.AcmiFile:write("LatitudeOffset="..self.LatitudeOffset.."\n");
			self.AcmiFile:write("LongitudeOffset="..self.LongitudeOffset.."\n");
			self.AcmiFile:write("FlightStartTime="..LoGetMissionStartTime().."\n");

			-- Pilot info
			local PlayerID=LoGetPlayerPlaneId();

			if PlayerID then
				self.AcmiFile:write(string.format("LocalPilotPlaneID=%x\n",self.GetOptimizedID(PlayerID)));
			end

			local PlayerName=LoGetPilotName();

			if PlayerName then
				self.AcmiFile:write("LocalPilotName="..self.CleanupName(PlayerName).."\n");
			end
		end
	end,

	-- Update ACMI log
	UpdateLog=function(self)

		-- Check parameters
		if not AcmiData.AcmiFile then
			return;
		end

		-- Write frame time
		local CurrentTime=LoGetModelTime();
		local FormatedTime;

		FormatedTime=string.format("#%.2f\n",CurrentTime);

		local ShouldUpdateLog=false;

		if CurrentTime>=self.NextUpdateTime then
			ShouldUpdateLog=true;
			self.NextUpdateTime=CurrentTime+self.UpdatePeriod;
		end

		-- Get world objects
		local WorldObjects=LoGetWorldObjects();

		if not WorldObjects then
			return;
		end

		-- ADD/UPDATE OBJECTS
		local IsNewObject;
		local OptimizedID;

		-- Player Plane Info
		local PlayerPlaneID=LoGetPlayerPlaneId();
		local PlayerPitch,PlayerRoll,PlayerYaw=LoGetADIPitchBankYaw();

		for ID,Object in pairs(WorldObjects) do

			-- Reduce ID size (to compact text log)
			OptimizedID=self.GetOptimizedID(ID);

			-- ADD NEW OBJECT
			if not self.Objects[ID] then

				if FormatedTime then
					self.AcmiFile:write(FormatedTime);
					FormatedTime=nil;
				end

				local ObjectName=self.CleanupName(Object.Name);

				if ID==PlayerPlaneID then
					self.AcmiFile:write(string.format("+%x,%u,%s,%s,%s\n",OptimizedID,Object.Subtype,Object.Coalition,Object.Country,ObjectName));
				else
					self.AcmiFile:write(string.format("+%x,%u,%s,%s,%s,rp\n",OptimizedID,Object.Subtype,Object.Coalition,Object.Country,ObjectName));
				end

				IsNewObject=true;
			else
				IsNewObject=false;
			end

			-- Log object dynamic properties
			local ObjectPrevValues=self.Objects[ID];
			local Log=string.format("%x",OptimizedID);
			local ChangeDetected=false;

			-- Latitude
			if IsNewObject==true or Object.LatLongAlt.Lat~=ObjectPrevValues.LatLongAlt.Lat then
				Log=Log..string.format(",%.6f",Object.LatLongAlt.Lat-self.LatitudeOffset);
				ChangeDetected=true;
			else
				Log=Log..",";
			end

			-- Longitude
			if IsNewObject==true or Object.LatLongAlt.Long~=ObjectPrevValues.LatLongAlt.Long then
				Log=Log..string.format(",%.6f",Object.LatLongAlt.Long-self.LongitudeOffset);
				ChangeDetected=true;
			else
				Log=Log..",";
			end

			-- Altitude
			if IsNewObject==true or Object.LatLongAlt.Alt~=ObjectPrevValues.LatLongAlt.Alt then
				Log=Log..string.format(",%.2f",Object.LatLongAlt.Alt);
				ChangeDetected=true;
			else
				Log=Log..",";
			end

			-- Roll (not yet available)
			local Roll=0;

			if PlayerPlaneID and ID==PlayerPlaneID then
				Roll=PlayerRoll;
			end

			if IsNewObject==true or Roll~=ObjectPrevValues.Roll then
				Log=Log..string.format(",%.1f",math.mod(math.deg(Roll),360));
				ChangeDetected=true;
			else
				Log=Log..",";
			end

			-- Pitch (not yet available)
			local Pitch=0;

			if PlayerPlaneID and ID==PlayerPlaneID then
				Pitch=-PlayerPitch;
			end

			if IsNewObject==true or Pitch~=ObjectPrevValues.Pitch then
				Log=Log..string.format(",%.1f",math.mod(math.deg(Pitch),360));
				ChangeDetected=true;
			else
				Log=Log..",";
			end

			-- Yaw (not yet available, emulated by using heading)
			local Yaw=Object.Heading;

			--Player Yaw is not reliable enough for an ACMI
			--if PlayerPlaneID and ID==PlayerPlaneID then
			--	Yaw=math.rad(360)-PlayerYaw;
			--end

			if IsNewObject==true or Yaw~=ObjectPrevValues.Yaw then
				Log=Log..string.format(",%.1f\n",math.mod(math.deg(Yaw),360));
				ChangeDetected=true;
			else
				Log=Log.."\n";
			end

			-- Log data
			if IsNewObject==true or ( ChangeDetected==true and ShouldUpdateLog==true ) then

				if FormatedTime then
					self.AcmiFile:write(FormatedTime);
					FormatedTime=nil;
				end

				self.AcmiFile:write(Log);

				-- Remember objet properties
				self.Objects[ID]=Object;
				self.Objects[ID].Roll=Roll;
				self.Objects[ID].Pitch=Pitch;
				self.Objects[ID].Yaw=Yaw;
			end
		end

		-- REMOVE OBSOLETE OBJECTS
		for ID,Object in pairs(self.Objects) do
			if not WorldObjects[ID] then

				if FormatedTime then
					self.AcmiFile:write(FormatedTime);
					FormatedTime=nil;
				end

				self.AcmiFile:write(string.format("-%x\n",self.GetOptimizedID(ID)));
				self.Objects[ID]=nil;
			end
		end
	end,

	-- Stop ACMI logging
	EndLog=function(self)
		if self.AcmiFile then
			io.close(self.AcmiFile)
		end
	end,
}

-- Works once just before mission start.
do
	local PrevLuaExportStart=LuaExportStart;

	LuaExportStart=function()
		AcmiData:BeginLog();

		if PrevLuaExportStart then
			PrevLuaExportStart();
		end
	end
end

-- Works just after every simulation frame.
do
	local PrevLuaExportAfterNextFrame=LuaExportAfterNextFrame;

	LuaExportAfterNextFrame=function()
		AcmiData:UpdateLog();

		if PrevLuaExportAfterNextFrame then
			PrevLuaExportAfterNextFrame();
		end
	end
end

-- Works once just after mission stop.
do
	local PrevLuaExportStop=LuaExportStop;

	LuaExportStop=function()
		AcmiData:EndLog();

		if PrevLuaExportStop then
			PrevLuaExportStop();
		end
	end
end
